/* --COPYRIGHT--,BSD
 * Copyright (c) 2014, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <driverlib.h>
#include <captivate.h>
#include <Thermostat_UI_Demo.h>
#include <Thermostat_UI_LEDs.h>

//*****************************************************************************
// System defines
//*****************************************************************************

#define MCLK_FREQ                                                     (8000000)
#define SMCLK_FREQ                                                    (2000000)
#define ACLK_FREQ                                                       (32768)
#define FLLREF_FREQ                                                     (32768)
#define FLL_RATIO                                     (MCLK_FREQ / FLLREF_FREQ)

//*****************************************************************************
// Capacitive touch Key defines
//*****************************************************************************

#define THERMOSTATUI_UP_KEY      (thermostatKeypad_E03)
#define THERMOSTATUI_UP_KEY_ID      (3)

#define THERMOSTATUI_DOWN_KEY    (thermostatKeypad_E07)
#define THERMOSTATUI_DOWN_KEY_ID    (7)

#define THERMOSTATUI_SET_KEY     (thermostatKeypad_E02)
#define THERMOSTATUI_SET_KEY_ID     (2)

#define THERMOSTATUI_HOLD_KEY    (thermostatKeypad_E06)
#define THERMOSTATUI_HOLD_KEY_ID    (6)


#define THERMOSTATUI_FANAUTO_KEY (thermostatKeypad_E01)
#define THERMOSTATUI_FANAUTO_KEY_ID (1)

#define THERMOSTATUI_FANON_KEY   (thermostatKeypad_E05)
#define THERMOSTATUI_FANON_KEY_ID   (5)


#define THERMOSTATUI_HEAT_KEY    (thermostatKeypad_E00)
#define THERMOSTATUI_HEAT_KEY_ID    (0)

#define THERMOSTATUI_COOL_KEY    (thermostatKeypad_E04)
#define THERMOSTATUI_COOL_KEY_ID    (4)

//*****************************************************************************
// external globals
//*****************************************************************************

extern tElement thermostatKeypad_E00;
extern tElement thermostatKeypad_E01;
extern tElement thermostatKeypad_E02;
extern tElement thermostatKeypad_E03;
extern tElement thermostatKeypad_E04;
extern tElement thermostatKeypad_E05;
extern tElement thermostatKeypad_E06;
extern tElement thermostatKeypad_E07;

//*****************************************************************************
// FRAM globals - Stored in FRAM to maintain state through power-loss
//*****************************************************************************

//! \brief ThermostatUI_fanState stores the current state of the fan that
//!        is controlled by the thermostat UI
//!
#pragma PERSISTENT(ThermostatUI_fanState)
#pragma LOCATION(ThermostatUI_fanState, 0x1800)
FanState ThermostatUI_fanState = FanState_Off;

//! \brief ThermostatUI_unitState stores the current state of the heating/cooling
//!        unit that is controlled by the thermostat UI
//!
#pragma PERSISTENT(ThermostatUI_unitState)
#pragma LOCATION(ThermostatUI_unitState, 0x1802)
UnitState ThermostatUI_unitState = UnitState_Off;

//! \brief ThermostatUI_hold allows for temporary thermostat settings to be
//!        used rather than "programmed" settings
//!
#pragma PERSISTENT(ThermostatUI_hold)
#pragma LOCATION(ThermostatUI_hold, 0x1806)
bool ThermostatUI_hold = false;

//! \brief ThermostatUI_holdUnitState stores the state of the heating/cooling
//!        unit while hold is active
//!
#pragma PERSISTENT(ThermostatUI_holdUnitState)
#pragma LOCATION(ThermostatUI_holdUnitState, 0x1808)
UnitState ThermostatUI_holdUnitState = UnitState_Off;

//! \brief ThermostatUI_holdFanState stores the state of the fan
//!        unit while hold is active
//!
#pragma PERSISTENT(ThermostatUI_holdFanState)
#pragma LOCATION(ThermostatUI_holdFanState, 0x180A)
FanState ThermostatUI_holdFanState = FanState_Off;


//*****************************************************************************
// Function Prototypes
//*****************************************************************************

//! \brief ThermostatUI_initIO(void) initializes Digital I/Os for the thermostat
//!        application
//!
static void ThermostatUI_initIO(void);

//! \brief ThermostatUI_initClocks(void) sets up clocks for the thermostat application
//!
static void ThermostatUI_initClocks(void);

//! \brief ThermostatUI_keypadHandler handles application response to keypad events
//! \param tSensor* pSensor is a pointer to the keypad sensor structure on which
//!                 events are to be handled
//!
void ThermostatUI_keypadHandler(tSensor *pSensor);

//*****************************************************************************
// Function Definitions
//*****************************************************************************

void ThermostatUI_initSystem(void)
{
	ThermostatUI_initIO();
	ThermostatUI_initClocks();
#ifndef POWER_TESTING
	LEDs_updateStatus(ThermostatUI_fanState, ThermostatUI_unitState, ThermostatUI_hold);
#endif
	CAPT_registerCallback(&thermostatKeypad, &ThermostatUI_keypadHandler);
}

void ThermostatUI_initIO(void)
{
	// P1.0: LED6
	// P1.1: IRQ (OPEN DRAIN)
	// P1.2: UCB0 I2C SDA
	// P1.3: UCB0 I2C SCL
	// P1.4: UCA0 UART TXD
	// P1.5: UCA0 UART RXD
	// P1.6: LED8
	// P1.7: LED7
	P1OUT  = (GPIO_PIN0 | GPIO_PIN6 | GPIO_PIN7);
	P1DIR  = (GPIO_PIN0 | GPIO_PIN6 | GPIO_PIN7);
	P1SEL0 = (GPIO_PIN2 | GPIO_PIN3 | GPIO_PIN4 | GPIO_PIN5);
	P1SEL1 = (0);

	// P2.0: LED1
	// P2.1: LED5
	// P2.2: SYNC (SET OUTPUT LOW UNLESS USED)
	// P2.3: OUTPUT LOW
	// P2.4: LED4
	// P2.5: LED3
	// P2.6: LED2
	// P2.7: OUTPUT LOW
	P2OUT  =  (GPIO_PIN0 | GPIO_PIN1 | GPIO_PIN4 | GPIO_PIN5 |\
			   GPIO_PIN6);
	P2DIR  =  (GPIO_PIN0 | GPIO_PIN1 | GPIO_PIN2 | GPIO_PIN3 |\
			   GPIO_PIN4 | GPIO_PIN5 | GPIO_PIN6 | GPIO_PIN7);
	P2SEL0 =  (0);
	P2SEL1 =  (0);

	// P3.0: OUTPUT LOW
	// P3.1: OUTPUT LOW
	// P3.2: OUTPUT LOW
	P3OUT  =  (0);
	P3DIR  =  (GPIO_PIN0 | GPIO_PIN1 | GPIO_PIN2);
	P3SEL0 =  (0);
	P3SEL1 =  (0);

	//
	// Clear port lock
	//
	PM5CTL0 &= ~LOCKLPM5;
}

void ThermostatUI_initClocks(void)
{
	//
	// Initialize Clock Signals
	//
	CS_initClockSignal(CS_FLLREF, CS_REFOCLK_SELECT, CS_CLOCK_DIVIDER_1);
	CS_initClockSignal(CS_ACLK, CS_REFOCLK_SELECT, CS_CLOCK_DIVIDER_1);
	CS_initClockSignal(CS_MCLK, CS_DCOCLKDIV_SELECT, CS_CLOCK_DIVIDER_1);
	CS_initClockSignal(CS_SMCLK, CS_DCOCLKDIV_SELECT, CS_CLOCK_DIVIDER_4);

	//
	// Tune the DCO
	//
	CS_initFLLSettle((MCLK_FREQ/1000), FLL_RATIO);

	//
	// Wait for DCO fault flag to clear
	//
	while (CS_getFaultFlagStatus(CS_DCOFFG | CS_FLLULIFG))
	{
		CS_clearFaultFlag(CS_DCOFFG | CS_FLLULIFG);
	}
}

void ThermostatUI_keypadHandler(tSensor *pSensor)
{
	// Read in sensor params pointed to by tSensor pointer
	tButtonSensorParams *pParams;
	pParams = (tButtonSensorParams*)(pSensor->pSensorParams);

	// Respond to dominant element if touch exists (one-at-a-time response)
	switch(pParams->ui16DominantElement)
	{
		case THERMOSTATUI_UP_KEY_ID:
			if(THERMOSTATUI_UP_KEY.bTouch == true && ThermostatUI_hold)
			{
				if(pSensor->bSensorPrevTouch == false)
				{
					P1DIR |= BIT1;
				}
#ifndef POWER_TESTING
				// Turn on UP_LED to indicate touch
				UP_LED_ON;

				// Turn off DOWN and SET LEDs. Multi-touch
				// not allowed for application
				DOWN_LED_OFF;
				SET_LED_OFF;
#endif
			}
			break;

		case THERMOSTATUI_DOWN_KEY_ID:
			if(THERMOSTATUI_DOWN_KEY.bTouch == true && ThermostatUI_hold)
			{
				if(pSensor->bSensorPrevTouch == false)
				{
					P1DIR |= BIT1;
				}

#ifndef POWER_TESTING
				// Turn on DOWN_LED to indicate touch
				DOWN_LED_ON;

				// Turn off UP and SET LEDs. Multi-touch
				// not allowed for application
				UP_LED_OFF;
				SET_LED_OFF;
#endif
			}
			break;

		case THERMOSTATUI_SET_KEY_ID:
			if(THERMOSTATUI_SET_KEY.bTouch == true && ThermostatUI_hold)
			{
				if(pSensor->bSensorPrevTouch == false)
				{
					P1DIR |= BIT1;
				}

#ifndef POWER_TESTING
				// Turn on SET LED to indicate touch
				SET_LED_ON;

				// Turn off UP and SET LEDs. Multi-touch
				// not allowed for application
				UP_LED_OFF;
				DOWN_LED_OFF;
#endif
			}
			break;

		case THERMOSTATUI_FANON_KEY_ID:
			// Act on new-touch only for maintained switch
			if ((pSensor->bSensorTouch == true) && (pSensor->bSensorPrevTouch == false) && ThermostatUI_hold)
			{
				P1DIR |= BIT1;

				// Enable write to FRAM data section
				SysCtl_enableFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_DATA);

				// Update ThermostatUI_fanState
				if (ThermostatUI_fanState != FanState_On)
				{
					ThermostatUI_fanState = FanState_On;
				}
				else
				{
					ThermostatUI_fanState = FanState_Off;
				}

				// Disable write to FRAM
				SysCtl_protectFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_DATA);

#ifndef POWER_TESTING
				// Update LEDs to reflect change in retained state
				LEDs_updateStatus(ThermostatUI_fanState, ThermostatUI_unitState, ThermostatUI_hold);
#endif
			}
			break;

		case THERMOSTATUI_FANAUTO_KEY_ID:
			// Act on new-touch only for maintained switch
			if ((pSensor->bSensorTouch == true) && (pSensor->bSensorPrevTouch == false) && ThermostatUI_hold)
			{
				P1DIR |= BIT1;

				// Enable write to FRAM data section
				SysCtl_enableFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_DATA);

				// Update ThermostatUI_fanState
				if (ThermostatUI_fanState != FanState_Auto)
				{
					ThermostatUI_fanState = FanState_Auto;
				}
				else
				{
					ThermostatUI_fanState = FanState_Off;
				}

				// Disable write to FRAM
				SysCtl_protectFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_DATA);

#ifndef POWER_TESTING
				// Update LEDs to reflect change in maintained state
				LEDs_updateStatus(ThermostatUI_fanState, ThermostatUI_unitState, ThermostatUI_hold);
#endif
			}
			break;

		case THERMOSTATUI_HEAT_KEY_ID:
			// Act on new-touch only for maintained switch
			if ((pSensor->bSensorTouch == true) && (pSensor->bSensorPrevTouch == false) && ThermostatUI_hold)
			{
				P1DIR |= BIT1;

				// Enable write to FRAM data section
				SysCtl_enableFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_DATA);

				// Update ThermostatUI_unitState
				if (ThermostatUI_unitState != UnitState_Heat)
				{
					ThermostatUI_unitState = UnitState_Heat;
				}
				else
				{
					ThermostatUI_unitState = UnitState_Off;
				}

				// Disable write to FRAM
				SysCtl_protectFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_DATA);

#ifndef POWER_TESTING
				// Update LEDs to reflect change in maintained state
				LEDs_updateStatus(ThermostatUI_fanState, ThermostatUI_unitState, ThermostatUI_hold);
#endif
			}
			break;

		case THERMOSTATUI_COOL_KEY_ID:
			// Act on new-touch only for maintained switch
			if ((pSensor->bSensorTouch == true) && (pSensor->bSensorPrevTouch == false) && ThermostatUI_hold)
			{
				P1DIR |= BIT1;

				// Enable write to FRAM data section
				SysCtl_enableFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_DATA);

				// Update ThermostatUI_unitState
				if (ThermostatUI_unitState != UnitState_Cool)
				{
					ThermostatUI_unitState = UnitState_Cool;
				}
				else
				{
					ThermostatUI_unitState = UnitState_Off;
				}

				// Disable write to FRAM
				SysCtl_protectFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_DATA);

#ifndef POWER_TESTING
				// Update LEDs to reflect change in maintained state
				LEDs_updateStatus(ThermostatUI_fanState, ThermostatUI_unitState, ThermostatUI_hold);
#endif
			}
			break;

		case THERMOSTATUI_HOLD_KEY_ID:
			// Act on new-touch only for maintained switch
			if ((pSensor->bSensorTouch == true) && (pSensor->bSensorPrevTouch == false))
			{
				P1DIR |= BIT1;

				// Enable write to FRAM data section
				SysCtl_enableFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_DATA);
				// Toggle Hold state
				ThermostatUI_hold ^= true;

				if(ThermostatUI_hold == false)
				{
					ThermostatUI_holdFanState = ThermostatUI_fanState;
					ThermostatUI_holdUnitState = ThermostatUI_unitState;
					ThermostatUI_fanState = FanState_Off;
					ThermostatUI_unitState = UnitState_Off;
				}
				else
				{
					ThermostatUI_fanState = ThermostatUI_holdFanState;
					ThermostatUI_unitState = ThermostatUI_holdUnitState;
				}

				// Disable write to FRAM
				SysCtl_protectFRAMWrite(SYSCTL_FRAMWRITEPROTECTION_DATA);

#ifndef POWER_TESTING
				// Update LEDs to reflect change in maintained state
				LEDs_updateStatus(ThermostatUI_fanState, ThermostatUI_unitState, ThermostatUI_hold);
#endif
			}
			break;

		default:
			break;
	}

#ifndef POWER_TESTING
	// Turn-off LEDs of all untouched momentary switches
	if(THERMOSTATUI_UP_KEY.bTouch == false)
	{
		UP_LED_OFF;
	}

	if(THERMOSTATUI_DOWN_KEY.bTouch == false)
	{
		DOWN_LED_OFF;
	}

	if(THERMOSTATUI_SET_KEY.bTouch == false)
	{
		SET_LED_OFF;
	}
#endif

	P1DIR &= ~BIT1;

}


