//*****************************************************************************
// I2CMaster.c
//
// \version __I2CMASTER_LIB_BUILD_VERSION__
// Released on __I2CMASTER_LIB_BUILD_DATE__
//
//*****************************************************************************

#include "I2CMaster.h"

#if (I2CMASTER__ENABLE==true)

//*****************************************************************************
//
//! \typedef tI2CMasterState defines the states that the I2C master
//! driver may be in during operation.
//
//*****************************************************************************
typedef enum
{
	eI2CMasterIsClosed = 0,
	eI2CMasterIsIdle,
	eI2CMasterIsCommunicating
} tI2CMasterState;

typedef union
{
	uint8_t ui8AllFlags;
	struct
	{
		uint8_t ui8ErrorFlags : 2;
		uint8_t ui8StatusFlags : 2;
	};
	struct
	{
		uint8_t bSlaveNack : 1;
		uint8_t bClockLowTimeout : 1;
		uint8_t bStop : 1;
		uint8_t bStart : 1;
	};
} tI2CMasterFlags;

typedef struct
{
	tI2CMasterState state;
	tI2CMasterFlags flags;
	uint8_t *pData;
} tI2CMasterDriver;

//*****************************************************************************
//
// eUSCI_B I2C Registers
//
//*****************************************************************************
#define I2CMASTER__CTLW0	UCB0CTLW0

#define I2CMASTER__CTLW1	UCB0CTLW1

#define I2CMASTER__BRW		UCB0BRW

#define I2CMASTER__TBCNT	UCB0TBCNT

#define I2CMASTER__I2CSA	UCB0I2CSA

#define I2CMASTER__STATW	UCB0STATW

#define I2CMASTER__IE		UCB0IE

#define I2CMASTER__IFG		UCB0IFG

#define I2CMASTER__TXBUF	UCB0TXBUF

#define I2CMASTER__RXBUF	UCB0RXBUF

//*****************************************************************************
//
//! eUSCI_B I2C Shortcut Macros
//
//*****************************************************************************
#define I2CMASTER__CLEAR_RST                     (I2CMASTER__CTLW0 &= ~UCSWRST)
#define I2CMASTER__SET_RST                        (I2CMASTER__CTLW0 |= UCSWRST)
#define I2CMASTER__WRITEMODE                         (I2CMASTER__CTLW0 |= UCTR)
#define I2CMASTER__READMODE                         (I2CMASTER__CTLW0 &= ~UCTR)
#define I2CMASTER__START                          (I2CMASTER__CTLW0 |= UCTXSTT)
#define I2CMASTER__RESTART                                    (I2CMASTER_START)
#define I2CMASTER__STOP                           (I2CMASTER__CTLW0 |= UCTXSTP)

//*****************************************************************************
//
//! \var g_state specifies the current state of the driver.
//
//*****************************************************************************
static volatile tI2CMasterDriver g_driver =
{
	.state = eI2CMasterIsClosed
};

static tI2CMasterError I2CMaster_checkForErrors(void);

static inline void I2CMaster_prep(uint8_t ui8Addr, uint8_t *pData,
		uint8_t ui8Cnt);

static inline void I2CMaster_cleanup(void);

static inline tI2CMasterError I2CMaster_finish(void);

void I2CMaster_open(void)
{
	I2CMASTER__SET_RST;
	I2CMASTER__CTLW0 |= UCMODE_3 | UCMST | UCSSEL__SMCLK;
	I2CMASTER__CTLW1 = UCASTP_2 | UCCLTO_2;
	I2CMASTER__BRW = I2CMASTER__PRESCALER;
	g_driver.state = eI2CMasterIsIdle;
}

void I2CMaster_close(void)
{
	I2CMASTER__SET_RST;
	g_driver.state = eI2CMasterIsClosed;
}

tI2CMasterError I2CMaster_checkForErrors(void)
{
	if (g_driver.flags.bClockLowTimeout == true)
	{
		return eI2CMaster_busTimeoutError;
	}
	else if (g_driver.flags.bSlaveNack == true)
	{
		return eI2CMaster_slaveNackError;
	}
	else
	{
		return eI2CMaster_noError;
	}
}

inline void I2CMaster_prep(uint8_t ui8Addr, uint8_t *pData,
		uint8_t ui8Cnt)
{
	I2CMASTER__SET_RST;
	g_driver.state = eI2CMasterIsCommunicating;
	g_driver.flags.ui8AllFlags = 0;
	g_driver.pData = pData;
	I2CMASTER__I2CSA = ui8Addr;
	I2CMASTER__TBCNT = ui8Cnt;
	I2CMASTER__CLEAR_RST;
	I2CMASTER__IE = UCNACKIE | UCCLTOIE | UCSTPIFG;
}

inline void I2CMaster_cleanup(void)
{
	I2CMASTER__IE = 0;
	g_driver.state = eI2CMasterIsIdle;
}

inline tI2CMasterError I2CMaster_finish(void)
{
	while (g_driver.flags.bStop == false)
	{
		if (g_driver.flags.ui8ErrorFlags != 0)
		{
			I2CMaster_cleanup();
			return I2CMaster_checkForErrors();
		}
	}

	I2CMaster_cleanup();
	return I2CMaster_checkForErrors();
}

tI2CMasterError I2CMaster_writeBuffer(uint8_t ui8Addr, uint8_t *pData,
		uint8_t ui8Cnt)
{
	if (g_driver.state != eI2CMasterIsIdle)
	{
		return eI2CMaster_unavailableError;
	}

	I2CMaster_prep(ui8Addr, pData, ui8Cnt);

	I2CMASTER__WRITEMODE;
	I2CMASTER__START;
	while (!(I2CMASTER__IFG & UCTXIFG))
	{
		if (g_driver.flags.ui8ErrorFlags != 0)
		{
			I2CMaster_cleanup();
			return I2CMaster_checkForErrors();
		}
	}

	// Begin writing via ISR
	I2CMASTER__IE |= UCTXIE;

	return I2CMaster_finish();
}

tI2CMasterError I2CMaster_readBuffer(uint8_t ui8Addr, uint8_t *pData,
		uint8_t ui8Cnt)
{
	if (g_driver.state != eI2CMasterIsIdle)
	{
		return eI2CMaster_unavailableError;
	}

	I2CMaster_prep(ui8Addr, pData, ui8Cnt);
	I2CMASTER__READMODE;
	I2CMASTER__START;
	I2CMASTER__IE |= UCRXIE;

	return I2CMaster_finish();
}

tI2CMasterError I2CMaster_writeReg8(uint8_t ui8Addr, uint8_t ui8Reg,
		uint8_t *pData, uint8_t ui8Cnt)
{
	tI2CMasterError error;

	if (g_driver.state != eI2CMasterIsIdle)
	{
		return eI2CMaster_unavailableError;
	}

	I2CMaster_prep(ui8Addr, pData, ui8Cnt+1);

	I2CMASTER__WRITEMODE;
	I2CMASTER__START;
	while (!(I2CMASTER__IFG & UCTXIFG))
	{
		if (g_driver.flags.ui8ErrorFlags != 0)
		{
			I2CMaster_cleanup();
			return I2CMaster_checkForErrors();
		}
	}

	I2CMASTER__TXBUF = ui8Reg;
	I2CMASTER__IE |= UCTXIE;
	error = I2CMaster_finish();

	return error;
}

tI2CMasterError I2CMaster_readReg8(uint8_t ui8Addr, uint8_t ui8Reg,
		uint8_t *pData, uint8_t ui8Cnt)
{
	tI2CMasterError error;

	if (g_driver.state != eI2CMasterIsIdle)
	{
		return eI2CMaster_unavailableError;
	}

	I2CMaster_prep(ui8Addr, pData, ui8Cnt);

	// Start communication by sending a start condition
	// and the slave bus address, and ensuring the slave
	// does not nack.
	I2CMASTER__WRITEMODE;
	I2CMASTER__START;
	while (!(I2CMASTER__IFG & UCTXIFG))
	{
		if (g_driver.flags.ui8ErrorFlags != 0)
		{
			I2CMaster_cleanup();
			return I2CMaster_checkForErrors();
		}
	}

	// Transmit the slave sub-register to read from
	I2CMASTER__TXBUF = ui8Reg;
	while (!(I2CMASTER__IFG & UCTXIFG))
	{
		// Handle the i8Cnt==1 case, where the stop bit is sent prematurely
		if (g_driver.flags.bStop == true)
		{
			break;
		}
		if (g_driver.flags.ui8ErrorFlags != 0)
		{
			I2CMaster_cleanup();
			return I2CMaster_checkForErrors();
		}
	}

	// Read out the data from the slave
	g_driver.flags.ui8StatusFlags = 0;
	I2CMASTER__READMODE;
	I2CMASTER__START;
	I2CMASTER__IE |= UCRXIE;
	error = I2CMaster_finish();

	return error;
}

//*****************************************************************************
//
//! \def I2CMASTER__EUSCI_VECTOR defines the interrupt vector location
//! of the selected eUSCI_B I2C peripheral.
//
//! \def I2CMASTER__EUSCI_IV defines the interrupt vector of the selected
//! eUSCI_B I2C peripheral.
//
//*****************************************************************************
#if (I2CMASTER__EUSCI_B_PERIPHERAL == EUSCI_B0_BASE)
#define I2CMASTER__EUSCI_VECTOR     	(USCI_B0_VECTOR)
#define I2CMASTER__EUSCI_IV    	 	    (UCB0IV)
#else
#error INVALID_EUSCI_B_SELECTION
#endif

#pragma vector=I2CMASTER__EUSCI_VECTOR
__interrupt void I2CMaster_ISR(void)
{
	switch(__even_in_range(I2CMASTER__EUSCI_IV, USCI_I2C_UCBIT9IFG))
	{
		case USCI_I2C_UCALIFG:  break;
		case USCI_I2C_UCNACKIFG:
			I2CMASTER__STOP;
			g_driver.flags.bSlaveNack = true;
			break;
		case USCI_I2C_UCSTTIFG:
			g_driver.flags.bStart = true;
			break;
		case USCI_I2C_UCSTPIFG:
			g_driver.flags.bStop = true;
			break;
		case USCI_I2C_UCRXIFG0:
			*g_driver.pData++ = I2CMASTER__RXBUF;
			break;
		case USCI_I2C_UCTXIFG0:
			I2CMASTER__TXBUF = *g_driver.pData++;
			break;
		case USCI_I2C_UCBCNTIFG: break;
		case USCI_I2C_UCCLTOIFG:
			g_driver.flags.bClockLowTimeout = true;
			break;
		case USCI_I2C_UCBIT9IFG: break;
		default: break;
	}
	__bic_SR_register_on_exit(LPM3_bits);
}

#endif /* I2CMASTER__ENABLE==true */
