//****************************************************************************
//
//! TIDM-FRAM-THERMOSTAT (Hardware Revision 1.0, Jul. 2014)
//! main.c
//
//! \brief Application for interfacing TIDM-FRAM-THERMOSTAT with
//!        TIDM-CAPTIVATE-THERMOSTAT-UI
//
//! \author Texas Instruments
//! \author A0224638
//
//! \version Rev 1.0, 10/2015
//
//*****************************************************************************

//*****************************************************************************
// Includes
//*****************************************************************************
#include <msp_ts430fr4_golbal.h>
#include "I2CMaster.h"
#include "msp_ts430fr4_menu.h"

//*****************************************************************************
// Globals
//*****************************************************************************

//! \brief Determines whether or not timer is displayed on
//!        thermostat LCD
bool Timer_disp = false;

//! \brief Contains hour, minute, and second data
//!        for timer.
tTimerformat TimerData = {0,0,0};

//! \brief System mode: Active or Standby (LPM3)
tSystemStatus SystemStatus = eStandby;

//! \brief Determines the setting type that is displayed
//!        in menu of setting options
tSettingMode SettingMode = eAll;

//! \brief Thermostat mode set: Cool, Heat, Humidity, Fan
tTSMode	TSMode_set = eCool;

//! \brief Thermostat mode setting saved when Heat/Cool
//!        is set directly by UI: Cool, Heat, Humidity, Fan
tTSMode	TSMode_saved = eCool;

//! \brief Current fan setting: Level 0 - Level 4 or Auto
tFanLevel FanLevel = eL0;

//! \brief Fan setting when fan is turned on: Level 0 - Level 4
tFanLevel FanLevel_onSetting = eL1;

//! \brief Desired temperature in degrees (Scale determined by TempFormat)
uint8_t SettingTemp = 20;

//! \brief Format in which temperature is displayed: Fahrenheit or Celsius
tTempformat Tempformat = eDegC;

//! \brief Format of the clock displayed on LCD (12 or 24 hour)
tTimeformat Timeformat = e12hour;

//! \brief Swing setting
tSwingStatus SwingStatus = eSwingdisable;

//! \brief Battery voltage displayed
tBatteryvol BatteryVol = e100p;

//! \brief Counter used to shut off LED after
//!        timeout to save power
uint8_t Power_led_counter = 0;

//! \brief Buffer containing command to send to
//!        I2C slave
uint8_t ui8TxBuffer[32];

//! \brief Buffer used to store I2C data received from
//!        slave
uint8_t ui8RxBuffer[32];

//! \brief Records any errors that occur during I2C comms
tI2CMasterError error;

//*****************************************************************************
// Main
//*****************************************************************************

//! /brief Main function for thermostat application
//!        Updates Timer, Clock Time, and Temperature on LCD, and
//!        requests sensor status packet from touch panel over I2C.
//!
void main(void)
{
	// Initialize clocks, digital I/Os, LCD module
	// ADC, and RTC to run thermostat application
	TS_initialize();

	// Open I2C port as master
	I2CMaster_open();

	while(1)
	{
		// Enter LPM3 and wait for interrupt
		__bis_SR_register(LPM3_bits + GIE);

		// Update Time/Temperature on LCD after new measurement
		if(SystemStatus == eActive)
		{
			if(Timer_disp)
			{
				LCD_displayTimer(TimerData);
			}
			else
			{
				LCD_displayTime(tRTCData, Timeformat);
			}
			LCD_displayTemperature(Temperature,Tempformat);

		}

		// Request sensor packet from CapTIvate controller
		// if notified of new data.
		if(Menu_buttonPressFlag)
		{
			ui8TxBuffer[0] = 0x00;
			ui8TxBuffer[1] = 0x00;

			// Request and read sensor packet data from slave at address 0x0A
			error = I2CMaster_writeBuffer(0x0A, &ui8TxBuffer[0], 2);
			error = I2CMaster_readBuffer(0x0A, &ui8RxBuffer[0], 6);

			// See if new touch event occurred
			if((ui8RxBuffer[5] & BIT0) && !(ui8RxBuffer[5] & BIT1))
			{
				// Update LCD according to new touch event
				uint8_t dominantElement = ui8RxBuffer[2];
				Menu_updateLCD(dominantElement);
			}

			// Disable button press flag
			Menu_buttonPressFlag = false;
		}

	}
}

