//*****************************************************************************
//! msp_ts430fr4_golbal.c
//
//! \brief Global functions and variables for thermostat LCD example
//!
//! \author Texas Instruments
//! \author A0224638
//
//! \version Rev 1.0
//
//*****************************************************************************

#include <msp_ts430fr4_golbal.h>

//*****************************************************************************
// Globals
//
//*****************************************************************************

//! \brief MONTH_Table contains the number of days in each month
//!
uint8_t const MONTH_Table[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

//! \brief Structure containing initial date and time settings
//!
tRTCformat tRTCData = {15, 9, 29, 12, 00, 00, eMonday};

//*****************************************************************************
//
//! \brief Perform GPIO initialize. Set all GPIOs to outputs and set low, except P2.2,
//!        P5.3, and P5.2. P2.2 is used as an interrupt request line from slave MCU
//!        controlling UI. P5.2 and P5.3 are set for I2C master comms.
//!
//! \param none
//! \return none
//
//*****************************************************************************
void GPIO_config(void)
{
	P1OUT &= ~(BIT0 | BIT1 | BIT2 | BIT3 | BIT4 | BIT5 | BIT6 | BIT7);
	P1DIR |= BIT0 | BIT1 | BIT3 | BIT4 | BIT5 | BIT6 | BIT7;

	// Set P1.2 as input with Pull-up resistor
	KEY_PORT_DIR &= ~BIT2;
	KEY_PORT_OUT |= BIT2;
	KEY_PORT_REN |= BIT2;

	// Clear P1.2 interrupt flag, set interrupt to falling-edge
	// and enable.
	KEY_PORT_IFG &= ~BIT2;
	KEY_PORT_IES |= BIT2;
	KEY_PORT_IE |= BIT2;

	P2OUT &= ~(BIT0 | BIT1 | BIT2 | BIT3 | BIT4 | BIT5 | BIT6 | BIT7);
	P2DIR |= BIT0 | BIT1 | BIT2 | BIT3 | BIT4 | BIT5 | BIT6 | BIT7;

	P3OUT &= ~(BIT0 | BIT1 | BIT2 | BIT3 | BIT4 | BIT5 | BIT6 | BIT7);
	P3DIR |= BIT0 | BIT1 | BIT2 | BIT3 | BIT4 | BIT5 | BIT6 | BIT7;

	P4OUT &= ~(BIT0 | BIT1 | BIT2 | BIT3 | BIT4 | BIT5 | BIT6 | BIT7);
	P4DIR |= BIT0 | BIT1 | BIT2 | BIT3 | BIT4 | BIT5 | BIT6 | BIT7;

	P5OUT &= ~(BIT0 | BIT1 | BIT4 | BIT5 | BIT6 | BIT7);
	P5DIR |= BIT0 | BIT1 | BIT4 | BIT5 | BIT6 | BIT7;

	// Configure P5.2 and P5.3 for I2C operation
	P5SEL0 |= BIT2 | BIT3;

	P6OUT &= ~(BIT0 | BIT1 | BIT2 | BIT3 | BIT4 | BIT5 | BIT6 | BIT7);
	P6DIR |= BIT0 | BIT1 | BIT2 | BIT3 | BIT4 | BIT5 | BIT6 | BIT7;

	P7OUT &= ~(BIT0 | BIT1 | BIT2 | BIT3 | BIT4 | BIT5 | BIT6 | BIT7);
	P7DIR |= BIT0 | BIT1 | BIT2 | BIT3 | BIT4 | BIT5 | BIT6 | BIT7;

	P8OUT &= ~(BIT0 | BIT1 | BIT2 | BIT3);
	P8DIR |= BIT0 | BIT1 | BIT2 | BIT3;
}

//*****************************************************************************
//
//! \brief Perform System clock initialize.
//!        ACLK = XT1CLK = 32.738kHz  MCLK = SMCLK = default ~ 1.04MHz
//!
//! \param none
//! \return none
//
//*****************************************************************************
void CLOCK_config()
{

	P4SEL0 |= BIT1 | BIT2;                  // Select pin4.1 pin4.2 as crystal function
	do
	{
		CSCTL7 &= ~(XT1OFFG + DCOFFG);    	// Clear XT1 and DCO fault flag
		SFRIFG1 &= ~OFIFG;
	}while (SFRIFG1 & OFIFG);             	// Test oscillator fault flag

	CSCTL4 = SELA__XT1CLK;					// IMPORTANT. Select ACLK = XT1 = 32768Hz
}

//*****************************************************************************
//
//! \brief Perform LCD initialize.
//!
//! \param none
//! \return none
//
//*****************************************************************************
void LCD_config()
{
	uint8_t i;
	uint8_t *pLCD = (uint8_t *)&LCDM0;
	for (i = 16; i > 0; i--)			// Clear LCD memory
		*pLCD++ = 0x00;					// CLR LCD

	LCDM14 = BIT3 | BIT6;				// COM0 -- L21
	LCDM15 = BIT1 | BIT4;				// COM1 -- L23

	SYSCFG2 |= LCDPCTL;      			// R13/R23/R33/LCDC1/LCDC2 pins selected

	LCDPCTL0 = 0xFFFF;
	LCDPCTL1 = 0xFFFF;					// L0~L27 pins selected

	LCDVCTL = LCDCPFSEL0 | LCDCPFSEL1 | LCDCPFSEL2 | LCDCPFSEL3 | VLCD_12 | LCDCPEN | LCDREFEN;

	LCDCSSEL0 = 0x0000;					// Configure COMs and SEGs
	LCDCSSEL1 = 0xF000;

	LCDCTL0 |= LCD4MUX | LCDLP;	// 4-mux selected
}

//*****************************************************************************
//
//! \brief Perform ADC initialize.
//!        Clock source MODCLK / Channel A13 Internal reference 1.5V
//!        Reference AVCC+ AVSS- / 50ksps
//!        10-bit resolution
//
//! \param none
//! \return none
//
//*****************************************************************************
void ADC10_config()
{
	SYSCFG2 |= ADCPCTL8;
	ADCCTL0 = ADCSHT_1 | ADCON;         // 8 ADCCLK cycles
	ADCCTL1 = ADCSHS_0 | ADCSHP | ADCSSEL_2 | ADCCONSEQ_0;// + ADCDIV_3;  // ADCSC bit SHS,
															// SMCLK
															// SC-SC
	ADCCTL2 |= ADCRES | ADCSR;          // 10 bit resolution, 50ksps
	ADCMCTL0 = ADCSREF_1 | ADCINCH_8;  // Reference Vref+/AVSS-, Channel 13
	ADCIE |= ADCIE0;                    // Interrupt enable
}

//*****************************************************************************
//
//! \brief Turn off the internal reference, output reference, and the ADC.
//!
//! \param none
//! \return none
//
//*****************************************************************************
void ADC_turnOff()
{
	PMMCTL0 = SVSHE | PMMPW;
	PMMCTL2 &= ~(EXTREFEN | INTREFEN);   // For low power
	ADCCTL0 &= ~ADCON;
}

//*****************************************************************************
//
//! \brief Turn on the internal reference, output reference, and ADC.
//!
//! \param none
//! \return none
//
//*****************************************************************************
void ADC_turnOn()
{
	PMMCTL0 = SVSHE | PMMPW;
	PMMCTL2 |= EXTREFEN | INTREFEN;
	ADCCTL0 |= ADCON;
	__delay_cycles(15);
}

//*****************************************************************************
//
//! \brief Perform RTC initialize. Clock source XT1CLK
//!
//! \param none
//! \return none
//
//*****************************************************************************
void RTC_config()
{
	RTCCTL = RTCSS__XT1CLK | RTCSR | RTCPS__1 | RTCIE;
	RTCMOD = 32768-1;		             //1s
}

//*****************************************************************************
//
//! \brief Perform Water Meter board initialize.
//!
//! \param none
//! \return None.
//
//*****************************************************************************
void TS_initialize(void)
{
	WDTCTL = WDTPW | WDTHOLD;  	// Stop watchdog

	PM5CTL0 &= ~LOCKLPM5;      	// Disable the GPIO power-on default high-impedance mode
								// to activate previously configured port settings

	// Setup GPIOs, System Clock, LCD module
	// RTC and ADC10
	GPIO_config();
	CLOCK_config();
	LCD_config();
	RTC_config();
	ADC10_config();
}

//*****************************************************************************
//
//! Real-time Clock interrupt service routine
//! used to update Time/Timer and measure temperature
//
//*****************************************************************************
#pragma vector=RTC_VECTOR
__interrupt void RTC_ISR(void)
{
	switch(__even_in_range(RTCIV,RTCIV_RTCIF))
	{
	case RTCIV_NONE: break;             // No interrupt

	case RTCIV_RTCIF:                   // RTC Overflow
	{
		uint8_t Days = 31;

		tRTCData.SECOND++;
		if(Power_led_counter > 0)
		{
			Power_led_counter--;
			if(Power_led_counter == 0)		// Turn off the power LED 5 seconds after power-on to save energy
				PWLED_OFF();
		}

		if(SystemStatus == eActive)		// Measure temperarute once a second
		{
			if(tRTCData.SECOND % 10 == 0)
			{
				ADC_turnOn();
				ADCCTL0 |= ADCENC + ADCSC;
			}
		}

		if (tRTCData.SECOND >= 60)       // 60s
		{
			tRTCData.SECOND = 0;
			tRTCData.MINUTE++;
			if (tRTCData.MINUTE >= 60)
			{
				tRTCData.MINUTE = 0;
				tRTCData.HOUR++;
			}

			if (tRTCData.HOUR >= 24)
			{
				tRTCData.HOUR = 0;
				tRTCData.DAY++;
				if (tRTCData.MONTH == 2)
				{
					if (tRTCData.YEAR % 4 == 0)
					{
						Days = 29;
					}
					else
					{
						Days = 28;
					}
				}
				else
				{
					Days = MONTH_Table[tRTCData.MONTH - 1];
				}
				tRTCData.Weekday++;
				if(tRTCData.Weekday > eSaturday)
				{
					tRTCData.Weekday = eSunday;
				}
			}

			if (tRTCData.DAY > Days)
			{
				tRTCData.DAY = 1;
				tRTCData.MONTH++;
			}

			if (tRTCData.MONTH > 12)
			{
				tRTCData.MONTH = 1;
				tRTCData.YEAR++;
			}
		}

		// Update timer if not at 00:00:00
		if(SystemStatus == eActive && (TimerData.SECOND > 0 || TimerData.MINUTE > 0 || TimerData.HOUR > 0))
		{
			TimerData.SECOND--;
			if(TimerData.SECOND < 0)
			{
				TimerData.SECOND = 59;
				TimerData.MINUTE--;
			}

			if(TimerData.MINUTE < 0)
			{
				TimerData.MINUTE = 59;
				TimerData.HOUR--;
			}
		}
		__low_power_mode_off_on_exit();
		break;
	}
	default:
		break;
	}
}
