//****************************************************************************************
//! msp_ts430fr4_lcd.c
//
//! \brief Functions to support lcd display.  Some
//! 	   functions are a direct access of the LCD_E registers while others take
//!        the definitions (see msp_ts430fr4_lcd.h) and place the approriate values in the
//!        LCD_E peripheral.
//
//! \author Texas Instruments
//! \author A0222632
//
//! \version Release 1
//
//
//****************************************************************************************
#include <msp_ts430fr4_golbal.h>

//****************************************************************************************
// Globals
//****************************************************************************************

//! \brief Constant array used to display charachters on LCD
//!
const uint8_t LCD_Tab[] = {
		a + b + c + d + e + f,			// Displays "0"
		b + c,							// Displays "1"
		a + b + d + e + g,				// Displays "2"
		a + b + c + d + g,				// Displays "3"
		b + c + f + g,					// Displays "4"
		a + c + d + f +g,				// Displays "5"
		a + c + d + e + f + g,			// Displays "6"
		a + b + c,						// Displays "7"
		a + b + c + d + e + f + g,		// Displays "8"
		a + b + c + d + f + g,			// Displays "9"
		g,								// Displays "-"
		b,								// Displays "'"
		0								// Displays " "
};

//****************************************************************************************
//
//! \brief Perform display all informaiton on lcd.
//!
//! \param None
//! \return None
//
//****************************************************************************************
void LCD_displayAll()
{
	uint8_t i;
	uint8_t *pLCD = (uint8_t *)&LCDM0;
	for (i = 14; i > 0; i--)			// Clear LCD memory
		*pLCD++ = 0xFF;					// CLR LCD
}

//****************************************************************************************
//
//! \brief Perform clear lcd display.
//!
//! \param None
//! \return None
//
//****************************************************************************************
void LCD_displayClear()
{
	uint8_t i;
	uint8_t *pLCD = (uint8_t *)&LCDM0;
	for (i = 14; i > 0; i--)			// Clear LCD memory
		*pLCD++ = 0x00;					// CLR LCD
}

//****************************************************************************************
//
//! \brief Perform display days of week informaiton on lcd.
//!
//! \param Weekday - day of week to display
//!
//! \return None
//
//****************************************************************************************
void LCD_displayWeekday(tWeekday Weekday)
{
	switch(Weekday)
	{
	case eSunday:
		LCDM5 = Sunday;
		break;
	case eMonday:
		LCDM5 = Monday;
		break;
	case eTuesday:
		LCDM5 = Tuesday;
		break;
	case eWednesday:
		LCDM5 = Wednesday;
		break;
	case eThursday:
		LCDM5 = Thursday;
		break;
	case eFriday:
		LCDM5 = Friday;
		break;
	case eSaturday:
		LCDM5 = Saturday;
		break;
	}
}
//****************************************************************************************
//
//! \brief Perform display system time informaiton.
//!
//
//! \param tData - Hour, minutes, and seconds to displya
//! \param tMode - 12 or 24 hour clock format
//!
//! \return None
//
//****************************************************************************************
void LCD_displayTime(tRTCformat tData, tTimeformat tMode)
{
	if(tMode == e12hour)
	{

		if(tData.HOUR < 12)
		{
			LCDM6 = LCD_Tab[tData.HOUR/10];
			LCDM7 = LCD_Tab[tData.HOUR%10];
			LCDM8 = LCD_Tab[tData.MINUTE/10];
			LCDM9 = LCD_Tab[tData.MINUTE%10]|DOTSEG;
		}
		else if(tData.HOUR == 12)
		{
			LCDM6 = LCD_Tab[tData.HOUR/10];
			LCDM7 = LCD_Tab[tData.HOUR%10];
			LCDM8 = LCD_Tab[tData.MINUTE/10]|DOTSEG;
			LCDM9 = LCD_Tab[tData.MINUTE%10];
		}
		else
		{
			LCDM6 = LCD_Tab[(tData.HOUR -12)/10];
			LCDM7 = LCD_Tab[(tData.HOUR -12)%10];
			LCDM8 = LCD_Tab[tData.MINUTE/10]|DOTSEG;
			LCDM9 = LCD_Tab[tData.MINUTE%10];
		}
	}
	else
	{
		LCDM6 = LCD_Tab[tData.HOUR/10];
		LCDM7 = LCD_Tab[tData.HOUR%10];
		LCDM8 = LCD_Tab[tData.MINUTE/10];
		LCDM9 = LCD_Tab[tData.MINUTE%10];
	}

	if(tData.SECOND%2 ==0)
		LCDM7 |= DOTSEG;
	else
		LCDM7 &= ~DOTSEG;

	LCD_displayWeekday(tData.Weekday);
}

//****************************************************************************************
//
//! \brief Perform display timer informaiton on lcd.
//!
//! \param tData - Timer data
//!
//! \return None
//
//****************************************************************************************
void LCD_displayTimer(tTimerformat tData)
{

	LCDM6 = LCD_Tab[tData.HOUR/10]|DOTSEG;
	LCDM7 = LCD_Tab[tData.HOUR%10]|DOTSEG;
	LCDM8 = LCD_Tab[tData.MINUTE/10];
	LCDM9 = LCD_Tab[tData.MINUTE%10];

}

//****************************************************************************************
//
//! \brief Perform display temperature informaiton on lcd.
//!
//! \param temp: Temperature value in degrees
//!	\param tMode: Temperature format C / F
//!
//! \return None
//
//****************************************************************************************
void LCD_displayTemperature(uint16_t temp, tTempformat tMode)
{
	if(tMode == eDegF)
	{
		temp = temp / 10 * 18 + 320;
		LCDM10 = (LCDM10 & 0xFC) | DegF;
	}
	else
		LCDM10 = (LCDM10 & 0xFC) | DegC;

	LCDM2 = LCD_Tab[temp / 100];
	LCDM3 = LCD_Tab[((temp % 100) / 10)] | DOTSEG;
	LCDM4 = LCD_Tab[temp % 10];
}

//****************************************************************************************
//
//! \brief Perform display setting temperature informaiton on lcd.
//!
//! \param temp: setting temperature
//!
//! \return None
//
//****************************************************************************************
void LCD_displayTempSet(uint8_t temp)
{
	LCDM12 = LCD_Tab[temp/10] | DOTSEG;
	LCDM13 = LCD_Tab[temp%10];
}

//****************************************************************************************
//
//! \brief Perform display battery voltage informaiton.
//!
//! \param tBattery: battery voltage level to be displayed
//!
//! \return None
//
//****************************************************************************************
void LCD_displayBatteryVol(tBatteryvol tBattery)
{
	switch(tBattery)
	{
	case e100p:
		LCDM10 = (LCDM10 & 0x07) | Bat_100p;
		break;
	case e75p:
		LCDM10 = (LCDM10 & 0x07) | Bat_75p;
		break;
	case e50p:
		LCDM10 = (LCDM10 & 0x07) | Bat_50p;
		break;
	case e25p:
		LCDM10 = (LCDM10 & 0x07) | Bat_25p;
		break;
	case e0p:
		LCDM10 = (LCDM10 & 0x07) | Bat_0p;
		break;
	}
}

//****************************************************************************************
//
//! Perform display Fan informaiton.
//!
//! \param tFan: fal level level to be displayed
//!
//! \return None
//
//****************************************************************************************
void LCD_displayFan(tFanLevel tFan)
{
	switch(tFan)
	{
	case eL0:
		LCDM1 = (LCDM1 & 0xE0) | Fan_L0;
		break;
	case eL1:
		LCDM1 = (LCDM1 & 0xE0) | Fan_L1;
		break;
	case eL2:
		LCDM1 = (LCDM1 & 0xE0) | Fan_L2;
		break;
	case eL3:
		LCDM1 = (LCDM1 & 0xE0) | Fan_L3;
		break;
	case eL4:
		LCDM1 = (LCDM1 & 0xE0) | Fan_L4;
		break;
	case eAuto:
		LCDM1 = (LCDM1 & 0xE0) | Fan_Auto;
		break;
	}
}

//****************************************************************************************
//
//! \brief Perform display operating mode informaiton.
//!
//! \param tMode: operating mode
//!
//! \return None
//
//****************************************************************************************
void LCD_displayTSMode(tTSMode tMode)
{
	switch(tMode)
	{
	case eHeat:
		LCDM11 = (LCDM11 & 0xF0) | TSM_Heat;
		break;
	case eCool:
		LCDM11 = (LCDM11 & 0xF0) | TSM_Cool;
		break;
	case eHumi:
		LCDM11 = (LCDM11 & 0xF0) | TSM_Humi;
		break;
	case eFann:
		LCDM11 = (LCDM11 & 0xF0) | TSM_Fan;
		break;
	}
}

//****************************************************************************************
//
//! \brief Perform display setting mode informaiton.
//!
//! \param tMode: setting mode
//!
//! \return None
//
//****************************************************************************************
void LCD_displaySettingMode(tSettingMode tMode)
{
	switch(tMode)
	{
	case eAll:
		LCDM0 = SettingMode_All;
		break;
	case eMode:
		LCDM0 = SettingMode_Mode;
		break;
	case eFan:
		LCDM0 = SettingMode_Fan;
		break;
	case eTemp:
		LCDM0 = SettingMode_Temp;
		break;
	case eTime:
		LCDM0 = SettingMode_Time;
		break;
	case e1224:
		LCDM0 = SettingMode_1224;
		break;
	case eTimer:
		LCDM0 = SettingMode_Timer;
		break;
	case eCF:
		LCDM0 = SettingMode_CF;
		break;
	case eSwing:
		LCDM0 = SettingMode_Swing;
		break;
	}
}

//****************************************************************************************
//
//! \brief Perform display swing status informaiton.
//!
//! \param SStatus: swing status
//!
//! \return None
//
//****************************************************************************************
void LCD_displaySwing(tSwingStatus SStatus)
{
	switch(SStatus)
	{
	case eSwingenable:
		LCDM11 = (LCDM11 & 0x7F) | 0x80;
		break;
	case eSwingdisable:
		LCDM11 = LCDM11 & 0x7F;
		break;
	}
}

//****************************************************************************************
//
//! \brief Perform display coomunication signal.
//!
//! \param OnOff: signal status
//!
//! \return None
//
//****************************************************************************************
void LCD_displaySignal(bool OnOff)
{
	switch(OnOff)
	{
	case true:
		LCDM11 = (LCDM11 & 0x8F) | 0x70;
		break;
	case false:
		LCDM11 = LCDM11 & 0x8F;
		break;
	}
}

//****************************************************************************************
//
//! \brief Perform LCD display test.
//!
//! \param None
//! \return None
//
//****************************************************************************************
void LCD_displayTest()
{
	LCD_displayClear();
	LCD_displayAll();
	LCD_displayClear();
}
