//*****************************************************************************
//! msp_ts430fr4_menu.c
//
//! \author Texas Instruments
//! \author A0224638
//
//! \version Rev 1.0
//
//*****************************************************************************

#include <msp_ts430fr4_golbal.h>
#include "msp_ts430fr4_menu.h"
#include "I2CMaster.h"

//*****************************************************************************
// Globals
//*****************************************************************************

//! \brief Gets set when an interrupt occurs from slave UI
bool Menu_buttonPressFlag = false;

//! \brief Set to true when FanAuto button is on
bool FanAuto_set = false;

//! \brief Set to true when FanOn button is on
bool FanOn_set = false;

//! \brief Set to true when Heat button is on
bool Heat_set = false;

//! \brief Set to true when Cool button is on
bool Cool_set = false;

//*****************************************************************************
// Functions
//*****************************************************************************

//*****************************************************************************
//
//! \brief Function used to update LCD when a button on the touch panel
//!        UI is pressed.
//!
//! \param pressedButton - Button that was recorded as dominant element when
//!                        touch event occured.
//!
//! \return None
//
//*****************************************************************************
void Menu_updateLCD(uint8_t pressedButton)
{
	// HOLD (Power) Button pressed
	if(pressedButton == 6)
	{
		// Respond according to system state
		switch(SystemStatus)
		{

		//If system was active (LCD on)
		case eActive:
			SystemStatus = eStandby;
			PWLED_OFF();
			LCD_Display_OFF();
			ADC_turnOff();
			GLED_OFF();
			RLED_OFF();
			break;

		//If system was in standby (LCD off)
		case eStandby:
			SystemStatus = eActive;
			SettingMode = eAll;
			Power_led_counter = 3;
			TimerData.HOUR = 0;
			TimerData.MINUTE = 0;
			TimerData.SECOND = 0;
			PWLED_ON();
			LCD_Display_ON();
			LCD_Display_LOGO();
			LCD_displayTime(tRTCData,Timeformat);
			LCD_displayTemperature(Temperature,Tempformat);
			LCD_displayBatteryVol(BatteryVol);
			LCD_displayTempSet(SettingTemp);
			LCD_displayFan(FanLevel);
			LCD_displayTSMode(TSMode_set);
			LCD_displaySettingMode(SettingMode);
			break;
		}
	}

	// SET Button pressed
	if(pressedButton == 2)
	{
		if(SystemStatus == eActive)
		{
			// Update which setting mode is
			// displayed on LCD
			switch(SettingMode)
			{
			case eAll:
				SettingMode = eMode;
				break;

			case eMode:
				if(!Heat_set && !Cool_set)
				{
					TSMode_set = TSMode_saved;
				}
				LCD_displayTSMode(TSMode_set);
				SettingMode = eFan;
				break;

			case eFan:
				if(FanOn_set)
				{
					// Update FanLevel to new setting
					FanLevel = FanLevel_onSetting;
				}
				// Display actual FanLevel on LCD
				LCD_displayFan(FanLevel);
				SettingMode = eTemp;
				break;

			case eTemp:
				SettingMode = eTime;
				break;

			case eTime:
				SettingMode = e1224;
				break;

			case e1224:
				SettingMode = eTimer;
				Timer_disp = true;
				LCD_displayTimer(TimerData);
				break;

			case eTimer:
				SettingMode = eCF;
				Timer_disp = false;
				LCD_displayTime(tRTCData,Timeformat);
				break;

			case eCF:
				SettingMode = eSwing;
				break;

			case eSwing:
				SettingMode = eAll;
				break;
			}
			LCD_displaySettingMode(SettingMode);
			// Show FanLevel for if fan was turned on
			if(SettingMode == eFan)
			{
				LCD_displayFan(FanLevel_onSetting);
			}
			if(SettingMode == eMode)
			{
				LCD_displayTSMode(TSMode_saved);
			}
		}
	}

	// UP Button pressed
	if(pressedButton == 3)
	{
		if(SystemStatus == eActive)
		{
			// Update LCD based on the current
			// setting mode.
			switch(SettingMode)
			{
			case eAll:
				break;

			case eMode:
				// Change thermostat mode
				if(TSMode_saved == eHeat)
					TSMode_saved = eFann;
				else
					TSMode_saved--;
				LCD_displayTSMode(TSMode_saved);
				break;

			case eFan:
				if(FanLevel_onSetting < eL4)
				{
					// Update Fan Level that would
					// be set if fan was on
					FanLevel_onSetting++;
					LCD_displayFan(FanLevel_onSetting);
				}
				break;

			case eTemp:
				// Increase temperature setting by a degree
				SettingTemp++;

				// Adjust to keep setting within bounds
				if(Tempformat == eDegC && SettingTemp > 32)
					SettingTemp = 32;
				else if(Tempformat == eDegF && SettingTemp > 90)
					SettingTemp = 90;
				LCD_displayTempSet(SettingTemp);
				break;
			case eTime:
				// Change time by 1 minute
				tRTCData.MINUTE++;
				if (tRTCData.MINUTE > 59)
					tRTCData.MINUTE = 0;
				LCD_displayTime(tRTCData,Timeformat);
				break;

			case e1224:
				// Switch time format from 12 hour to 24 hour clock
				Timeformat++;
				if(Timeformat > e24hour)
					Timeformat = e12hour;
				LCD_displayTime(tRTCData,Timeformat);
				break;

			case eTimer:
				// Change timer by 1 minute
				TimerData.MINUTE ++;
				TimerData.SECOND = 60;

				if(TimerData.MINUTE>=60)
				{
					TimerData.HOUR ++;
					TimerData.MINUTE -= 60;
				}

				if(TimerData.HOUR > 12)
					TimerData.HOUR = 0;
				LCD_displayTimer(TimerData);
				break;

			case eCF:
				switch(Tempformat)
				{
				case eDegC:
					// Switch temp format to degrees F
					Tempformat = eDegF;
					SettingTemp = (SettingTemp * 18 + 320)/10;
					break;
				case eDegF:
					// Switch temperature format to degrees C
					Tempformat = eDegC;
					SettingTemp = (SettingTemp*10 - 320)/18;
					if(SettingTemp > 32)
						SettingTemp = 32;
					else if(SettingTemp < 16)
						SettingTemp = 16;
					break;
				}
				LCD_displayTemperature(Temperature,Tempformat);
				LCD_displayTempSet(SettingTemp);
				break;

			case eSwing:
				// Change swing status
				SwingStatus++;
				if(SwingStatus > eSwingenable)
					SwingStatus = eSwingdisable;
				LCD_displaySwing(SwingStatus);
				break;
			}
		}
	}

	// DOWN Button pressed
	if(pressedButton == 7)
	{
		if(SystemStatus == eActive)
		{
			switch(SettingMode)
			{
			case eAll:
				break;

			case eMode:
				// Change thermostat mode
				TSMode_saved++;
				if(TSMode_saved > eFann)
					TSMode_saved = eHeat;
				LCD_displayTSMode(TSMode_saved);
				break;

			case eFan:
				if(FanLevel_onSetting > eL1)
					FanLevel_onSetting--;
				LCD_displayFan(FanLevel_onSetting);
				break;

			case eTemp:
				SettingTemp--;
				if(Tempformat == eDegC && SettingTemp < 16)
					SettingTemp = 16;
				else if(Tempformat == eDegF && SettingTemp < 60)
					SettingTemp = 60;
				LCD_displayTempSet(SettingTemp);
				break;

			case eTime:
				tRTCData.HOUR++;
				if (tRTCData.HOUR > 23)
					tRTCData.HOUR = 0;
				LCD_displayTime(tRTCData,Timeformat);
				break;

			case e1224:
				if(tRTCData.Weekday < eSaturday)
					tRTCData.Weekday++;
				else
					tRTCData.Weekday = eSunday;
				LCD_displayTime(tRTCData,Timeformat);
				break;

			case eTimer:
				TimerData.MINUTE--;
				if(TimerData.MINUTE < 0)
				{
					TimerData.HOUR --;
					TimerData.MINUTE += 60;
				}
				if(TimerData.HOUR < 0)
				{
					TimerData.HOUR = 0;
					TimerData.MINUTE = 0;
				}
				LCD_displayTimer(TimerData);
				break;

			case eCF:
				switch(Tempformat)
				{
				case eDegC:
					Tempformat = eDegF;
					SettingTemp = (SettingTemp * 18 + 320)/10;
					break;
				case eDegF:
					Tempformat = eDegC;
					SettingTemp = (SettingTemp*10 - 320)/18;
					if(SettingTemp > 32)
						SettingTemp = 32;
					else if(SettingTemp < 16)
						SettingTemp = 16;
					break;
				}
				LCD_displayTemperature(Temperature,Tempformat);
				LCD_displayTempSet(SettingTemp);
				break;

			case eSwing:
				// Change swing status
				SwingStatus++;
				if(SwingStatus > eSwingenable)
					SwingStatus = eSwingdisable;
				LCD_displaySwing(SwingStatus);
				break;
			}
		}
	}

	// FAN AUTO Button pressed
	if(pressedButton == 1)
	{
		if(FanAuto_set)
		{
			FanAuto_set = false;
			FanLevel = eL0;
			if(SettingMode != eFan)
			{
				LCD_displayFan(FanLevel);
			}
		}
		else
		{
			if(FanOn_set)
			{
				FanOn_set = false;
			}
			FanAuto_set = true;
			FanLevel = eAuto;
			if(SettingMode != eFan)
			{
				LCD_displayFan(FanLevel);
			}
		}
	}

	// FAN ON Button pressed
	if(pressedButton == 5)
	{
		if(FanOn_set)
		{
			FanOn_set = false;
			FanLevel = eL0;
			if(SettingMode != eFan)
			{
				LCD_displayFan(FanLevel);
			}
		}
		else
		{
			if(FanAuto_set)
			{
				FanAuto_set = false;
			}
			FanLevel = FanLevel_onSetting;
			FanOn_set = true;
			LCD_displayFan(FanLevel);
		}
	}

	// HEAT Button pressed
	if(pressedButton == 0)
	{
		if(Heat_set)
		{
			Heat_set = false;
			TSMode_set = TSMode_saved;
			LCD_displayTSMode(TSMode_set);
		}
		else
		{
			Heat_set = true;
			if(SettingMode == eMode)
			{
				TSMode_set = eHeat;
			}
			else
			{
				if(!Cool_set)
				{
					TSMode_saved = TSMode_set;
				}
				TSMode_set = eHeat;
				LCD_displayTSMode(TSMode_set);
			}

			if(Cool_set)
			{
				Cool_set = false;
			}

		}
	}

	// COOL Button pressed
	if(pressedButton == 4)
	{
		if(Cool_set)
		{
			Cool_set = false;
			TSMode_set = TSMode_saved;
			LCD_displayTSMode(TSMode_set);
		}
		else
		{
			Cool_set = true;
			if(SettingMode == eMode)
			{
				TSMode_set = eCool;
			}
			else
			{
				if(!Heat_set)
				{
					TSMode_saved = TSMode_set;
				}
				TSMode_set = eCool;
				LCD_displayTSMode(TSMode_set);
			}

			if(Heat_set)
			{
				Heat_set = false;
			}
		}
	}
}

//*****************************************************************************
// ISR
//*****************************************************************************

//*****************************************************************************
//
//! \brief Port 1 ISR - Used to set button pressed flag and wake CPU from LPM
//!                     when interrupt is requested from slave controlling UI
//!
//
//*****************************************************************************
#pragma vector=PORT1_VECTOR
__interrupt void PORT1_ISR(void)
{
	// Wake if interrupt from P1.2
	if(KEY_PORT_IFG & BIT2)
	{
		Menu_buttonPressFlag = true;
		__low_power_mode_off_on_exit();
	}

	KEY_PORT_IFG = 0;
}
